package errcode

import (
	"github.com/gofiber/fiber/v2"
)

// header error code ,start:100
var (
	ErrHeaderRequestId     = NewError(101, "Header Request-Id Not Found")
	ErrHeaderAuthorization = NewError(102, "Header Authorization Not Found")
	ErrHeaderSecretKey     = NewError(103, "Header Secret-Key Not Found")
)

// standard error code ,start:110
var (
	ErrRequestParse  = NewError(111, "Request Parse Fail")
	ErrRequestMust   = NewError(112, "Request Params Required")
	ErrPermission    = NewError(113, "Permission Denied")
	ErrJsonUnmarshal = NewError(114, "Json Unmarshal Fail")
	ErrJsonMarshal   = NewError(115, "Json Marshal Fail")
	ErrInternal      = NewError(116, "Internal Server Error")
)

// jwt error code ,start:130
var (
	ErrJWTAuthNotFound     = NewError(131, "JWT Authorization Not Found")
	ErrJWTBase64Decode     = NewError(132, "JWT Authorization Base64 Decode Error")
	ErrJWTAuthParseFail    = NewError(133, "JWT Authorization Fail")
	ErrJWTAuthKeyId        = NewError(134, "JWT Key:Id Incorrect")
	ErrJWTAuthKeyIdentity  = NewError(135, "JWT Key:Identity Incorrect")
	ErrJWTAuthTokenChanged = NewError(136, "JWT Authorization Changed")
	ErrJWTAuthExpire       = NewError(137, "JWT Authorization Expire")
	ErrJWTJsonDecode       = NewError(138, "JWT Authorization JSON Decode Error")
	ErrJWTJsonEncode       = NewError(139, "JWT Authorization JSON Encode Error")
)

// model error code ,start:150
var (
	ErrDB    = NewError(151, "DB Fatal Error")
	ErrRedis = NewError(152, "Redis Fatal Error")
	ErrMq    = NewError(153, "MQ Fatal Error")
	ErrOss   = NewError(154, "OSS Fatal Error")
	ErrRpc   = NewError(155, "RPC Fatal Error")
	ErrApm   = NewError(156, "APM Fatal Error")
	ErrEtcd  = NewError(157, "Etcd Fatal Error")
)

// google grpc error status.
var (
	OK                    = NewError(171, "OK")
	ErrCanceled           = NewError(172, "Canceled")
	ErrUnknown            = NewError(173, "Unknown")
	ErrInvalidArgument    = NewError(174, "Invalid Argument")
	ErrDeadlineExceeded   = NewError(175, "Deadline Exceeded")
	ErrAlreadyExists      = NewError(176, "Already Exists")
	ErrPermissionDenied   = NewError(177, "Permission Denied")
	ErrResourceExhausted  = NewError(178, "Resource Exhausted")
	ErrFailedPrecondition = NewError(179, "Failed Precondition")
	ErrAborted            = NewError(181, "Aborted")
	ErrOutOfRange         = NewError(182, "Out Of Range")
	ErrUnimplemented      = NewError(183, "Unimplemented")
	ErrUnavailable        = NewError(184, "Unavailable")
	ErrDataLoss           = NewError(185, "Data Loss")
	ErrUnauthenticated    = NewError(186, "Unauthenticated")
	ErrJSONMarshal        = NewError(187, "Marshal JSON")
	ErrJSONUnmarshal      = NewError(188, "Unmarshal JSON")
	ErrPasswd             = NewError(189, "Password Error")

	ErrSmsCode          = NewError(191, "SMS Code Invalid")
	ErrIdArgument       = NewError(192, "ID Invalid Argument")
	ErrIdentityArgument = NewError(193, "Identity Invalid Argument")
)

var (
	ErrBase64Decode     = NewError(201, "Auth Token Base64 Decode Error")
	ErrAuthNotFound     = NewError(202, "Auth Token Not Found")
	ErrAuthParseFail    = NewError(203, "Auth Parse Fail")
	ErrAuthId           = NewError(204, "Auth Id Not Passed")
	ErrAuthIdentity     = NewError(205, "Auth Identity Not Passed")
	ErrAuthTokenChanged = NewError(206, "Auth Token Changed")
	ErrAuthIdType       = NewError(207, "Auth Id Type Error")
	ErrAuthExpire       = NewError(208, "Auth Token Expire")
	ErrAuthClient       = NewError(209, "Auth Token Client Not Passed")
	ErrJsonDecode       = NewError(210, "Auth JSON Decode Error")
	ErrJsonEncode       = NewError(211, "Auth JSON Encode Error")
	ErrAuthSecret       = NewError(212, "Auth JwtSecret Error")
	ErrAccountNotFound  = NewError(213, "Account Not Found")
)

func NewError(code int, msg string) error {
	return fiber.NewError(code, msg)
}

// custom error,status code:500
func ErrFatal(msg string) error {
	return fiber.NewError(500, msg)
}

func ErrNotFound(msg string) error {
	return fiber.NewError(404, msg+" Not Found")
}
